@@ -25,7 +25,7 @@
 #include <init.h>
 #include <environment.h>
 #include <asm/armlinux.h>
-#include <asm/mach-types.h>
+#include <generated/mach-types.h>
 #include <partition.h>
 #include <fs.h>
 #include <fcntl.h>
@@ -88,6 +88,7 @@ static void ek_add_device_nand(void)
 }
 
 static struct device_d cfi_dev = {
+	.id		= -1,
 	.name		= "cfi_flash",
 	.map_base	= AT91_CHIPSELECT_0,
 	.size		= 8 * 1024 * 1024,
