@@ -146,7 +146,7 @@ int eth_register(struct eth_device *edev)
 	unsigned char ethaddr[6];
 
 	if (!edev->get_ethaddr) {
-		printf("no get_mac_address found for current eth device\n");
+		dev_err(dev, "no get_mac_address found for current eth device\n");
 		return -1;
 	}
 
@@ -167,7 +167,7 @@ int eth_register(struct eth_device *edev)
 
 	if (edev->get_ethaddr(edev, ethaddr) == 0) {
 		ethaddr_to_string(ethaddr, ethaddr_str);
-		printf("got MAC address from EEPROM: %s\n", ethaddr_str);
+		dev_info(dev, "got MAC address from EEPROM: %s\n", ethaddr_str);
 		dev_set_param(dev, "ethaddr", ethaddr_str);
 	}
 
